# gastempt1.r
# (C) 2004 Dieter Menne, http://www.menne-biomed.de
# Revised October 2007
# Vanilla fit of gastric emptying curves
# Main reference (short: PB)
# Pinheiro/Bates, Mixed-effects models in S an S-PLUS, Springer 2000
library(nlme) #
options(digits=3) # only print 3 significant digits

# Self starting functions used to fit gastric emptying curves
# See PB 8.1.2 how to construct it
EmptInit0= function(mCall,LHS,data)
{
  model=as.character(mCall[[1]])
  xy=sortedXyData(mCall[["t"]],LHS,data)
  if (nrow(xy) < 6) {
    .Exit("Too few distinct input values to fit tmodel")
  }
  # we use the first of the sorted data points for
  # the initial volume (v0) estimation
  v0=xy[1,2]
  xy$y=log(xy$y/v0)
  tempt=log(0.5)/coef(lm(y~x,data=xy))[2]
  kappa = 0.8 # a good starting value
  tempt=max(min(tempt/2,200),20) # empirical correction
  value = c(v0,kappa,tempt)
  names(value)=mCall[c("v0","kappa","tempt")]
  value
}

# Standard LinExp model
SSEmptLinExp=selfStart(~v0*(1+kappa*t/tempt)*exp(-t/tempt),
  initial=EmptInit0, parameters= c("v0","kappa","tempt"))

# Get Data comma-separated (csv) file with header
# Sort order of data is arbitrary
# Header in first row must read (discard the #) :
# subj,treat,t,v
ge = read.table("gastempt.csv",sep=",",header=TRUE)
# Make sure subject are not treated as numbers
ge$subj = as.factor(ge$subj)

# We first want separate fits for each subject/treatment combination,
# so we create a new factor <subj>.<treat>
# nlsList could handle the separate factors, but nlme cannot
# use it as an start value currently
ge$subjtreat = as.factor(paste(ge$subj,ge$treat,sep="."))
# See PB 8.1.3
gelist = nlsList(v~SSEmptLinExp(t,v0,kappa,tempt)|subjtreat,data=ge)
print(gelist)

# compute simple nlme fit
# pnlsTol default value of 1.e-3 is too small, will lead to oscillations
# http://finzi.psych.upenn.edu/R/Rhelp02a/archive/59955.html
# Try verbose=TRUE in case of non-convergence
# This is similar to the example in PB, page 368 (CO2)
contr = nlmeControl(pnlsTol=0.5)
ge0.nlme=nlme(gelist,control=contr,verbose=FALSE)
# If you just want the raw coefficients, simply do:
# print(coef(v0.nlme))
# We will do a bit more work with the coeffs, so we store them first
ge0.coef = coef(ge0.nlme)

# Determine t(50%) from tempt and kappa.
# No closed solution known, we use newton  approximation
t50 = function(x){
  value= uniroot(function(t)
     SSEmptLinExp(t,1,x$kappa,x$tempt) - 0.5,
     interval=c(20,2000)) # we expect between 10 and 2000 minutes
  value$root
}

# add half-emptying time to coefficients
ge0.coef$t50=NA
for (i in 1:nrow(ge0.coef)) {
  ge0.coef$t50[i]=t50(ge0.coef[i,])
}

print(ge0.coef)






